#!/usr/bin/python3

import matplotlib.pyplot as plt
import subprocess as sp
import argparse
import time
import threading
import psutil
import matplotlib as mpl
mpl.use('Agg')


def run_cmd(cmd, cycle, interval):
    for i in range(0, cycle):
        sp.call(cmd, shell=True)
        time.sleep(interval)


def record_log(pid):
    proc = psutil.Process(pid)
    log_file = "{0}-{1:d}".format(proc.name(), proc.pid)
    with open(log_file, 'a') as f:
        f.write("{0:d} {1:d}\n".format(
            int(time.time() - proc.create_time()), proc.memory_info().rss))


def record_draw(pid, unit):
    proc = psutil.Process(pid)
    log_file = "%s-%d" % (proc.name(), proc.pid)
    if unit in ['B', "Byte"]:
        rate = 1
    elif unit in ["K", "KiB"]:
        rate = 1024
    elif unit in ["M", "MiB"]:
        rate = 1024 ** 2
    elif unit in ["G", "GiB"]:
        rate = 1024 ** 3
    else:
        print("Wrong unit")
        return -1
    with open(log_file, 'r') as f:
        lines = f.read().splitlines()
        plt.figure(figsize=[12, 9], dpi=5000)
        plt.plot([int(i.split(' ')[0]) for i in lines],
                 [float(i.split(' ')[1]) / rate for i in lines],
                 lw=0.2)
        plt.xlabel("running time(s)")
        plt.ylabel("RSS of {0}({1})".format(proc.name(), unit))
        plt.savefig('{0}.svg'.format(log_file))
        plt.clf()
        plt.close()
    return 0


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('-e', '--execute', type=str, required=True,
                        help="The command to execute")
    parser.add_argument('-p', '--pids', type=str, required=True,
                        help="The pid list to monitor")
    parser.add_argument('-c', '--cycle', type=int, required=True,
                        help="The cycles of running command")
    parser.add_argument('-i', '--interval', type=float, default=1,
                        help="The interval to next command")
    parser.add_argument('-u', '--unit', type=str, default="MiB",
                        choices=['B', "Byte", "K", "KiB",
                                 "M", "MiB", "G", "GiB"],
                        help="The unit of RSS memory")
    parser.add_argument('-g', '--granularity', type=float, default=0.1,
                        help='The granularity of memory monitor')
    args = parser.parse_args()
    thread = threading.Thread(target=run_cmd, args=(
        args.execute, args.cycle, args.interval))
    pids = [int(i) for i in args.pids.split(' ') if i]
    thread.start()
    while thread.is_alive():
        for pid in pids:
            record_log(pid)
            time.sleep(args.granularity)

    thread.join()
    for pid in pids:
        record_draw(pid, args.unit)
